@@ -88,8 +88,12 @@ typedef struct {
 #define PROBE_ALL_GPUS_BOOL_OPTION              33
 #define DYNAMIC_TWINVIEW_BOOL_OPTION            34
 #define INCLUDE_IMPLICIT_METAMODES_BOOL_OPTION  35
+#define USE_EVENTS_BOOL_OPTION                  36
+#define CONNECT_TO_ACPID_BOOL_OPTION            37
+#define ENABLE_ACPI_HOTKEYS_BOOL_OPTION         38
 
-#define XCONFIG_BOOL_OPTION_COUNT (INCLUDE_IMPLICIT_METAMODES_BOOL_OPTION + 1)
+
+#define XCONFIG_BOOL_OPTION_COUNT (ENABLE_ACPI_HOTKEYS_BOOL_OPTION + 1)
 
 /* # of 32-bit variables needed to hold all the boolean options (bits) */
 #define XCONFIG_BOOL_OPTION_SLOTS  \
@@ -140,6 +144,7 @@ typedef struct __options {
     int nvagp;
     int transparent_index;
     int stereo;
+    int cool_bits;
 
     char *xconfig;
     char *output_xconfig;
@@ -150,11 +155,19 @@ typedef struct __options {
     char *rotate;
 
     char *nvidia_cfg_path;
-    char *extract_edids_from_log;
+    char *extract_edids_from_file;
     char *extract_edids_output_file;
     char *twinview_xinerama_info_order;
     char *logo_path;
     char *twinview_orientation;
+    char *use_display_device;
+    char *custom_edid;
+    char *tv_standard;
+    char *tv_out_format;
+    char *acpid_socket_path;
+    char *handle_special_keys;
+
+    double tv_over_scan;
 
     struct {
         int x;
@@ -162,6 +175,7 @@ typedef struct __options {
     } virtual;
 
     TextRows add_modes;
+    TextRows add_modes_list;
     TextRows remove_modes;
 
     GenerateOptions gop;
@@ -227,6 +241,7 @@ int update_modules(XConfigPtr config);
 int update_screen(Options *op, XConfigPtr config, XConfigScreenPtr screen);
 XConfigLayoutPtr get_layout(Options *op, XConfigPtr config);
 int update_extensions(Options *op, XConfigPtr config);
+int update_server_flags(Options *op, XConfigPtr config);
 
 /* multiple_screens.c */
 
